;;;; stack-support.scm
;;;; Kon Lovett, Mar 29 2005

(eval-when (compile)
	(declare
  	(usual-integrations)
		(fixnum)
		(inline)
    (no-procedure-checks)
    (no-bound-checks)
		(export
			s-u$node-ref
			stack-cut!
			stack-pop! ) ) )

;;;

(include "stack")

;;; Helpers

(define (s-u$node-ref loc s n)
	(when (or (not (integer? n)) (negative? n))
		(error loc "invalid stack index; must be non-negative integer" n))
	(let loop ([ns s] [nn n])
		(if (stack-empty? ns)
			  (error loc "empty stack; stack index too large" n)
        (let ([sn (cdr ns)])
          (if (zero? nn)
              sn
              (loop sn (sub1 nn)) ) ) ) ) )

;;;

;; STACK-CUT! Stack Start-Depth !#optional End-Depth
;;
;; Stack - the stack to cut
;;
;; Start-Depth - the depth to start the cut, 0 <= & <= (stack-depth Stack)
;;
;; End-Depth - the depth to end the cut, Start-Depth <= & <=
;; (stack-depth Stack), defaults to Start-Depth
;;
;; Returns a list of the cut stack elements. The stack is modified in place.

(define (stack-cut! s sd . r)
	(let ([ed (:optional r sd)])
		(cond [(or (not (integer? sd)) (negative? sd))
            (error 'stack-cut! "invalid start depth; must be non-negative integer" sd)]
          [(or (not (integer? sd)) (negative? ed))
            (error 'stack-cut! "invalid end depth; must be non-negative integer" ed)]
          [(< ed sd)
            (error 'stack-cut! "invalid interval" sd ed)])
		(let* ([ds (sub1 sd)]
					 [sn (if (negative? ds) s (s-u$node-ref 'stack-cut! s ds))]
					 [en (s-u$node-ref 'stack-cut! sn (- ed sd))]
					 [ls (cdr sn)])
			(set-cdr! sn (cdr en))
			(set-cdr! en '())
			ls ) ) )

;; STACK-POP! Stack
;;
;; Stack - the stack to pop
;;
;; Returns the top of the stack. The stack is modified in place.

(define (stack-pop! stk)
	(when (stack-empty? stk)
		(error 'stack-pop! "empty stack"))
	(stack-pop*! stk) )
